# File tres/Scripts/ctres.cl
# January 25, 2008
# By Doug Mink, SAO Telescope Data Center

# CTRES - Process TRES images from raw data files, removing cosmic rays, to spectra

procedure ctres (images)

string	images=""	{prompt="Images to process"}
string	imdate=""	{prompt="Date for images"}
string	instdir=""	{prompt="Directory containing date directory"}
bool	fixpreamp=yes	{prompt="Run tpreamp to fix time constant (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	flatten=yes	{prompt="Run TFLAT to Flatten image (yes or no)"}
bool	disperse=yes	{prompt="Dispersion correct extracted spectra (yes or no)"}
string	normfile=""	{prompt="Pixel to pixel tmakeflat normalization file"}
string	normdir="tresdata$"	{prompt="Directory for normalization file"}
bool	recenter=no	{prompt="Recenter orders during extraction (yes or no)"}
string	comproot="comp"	{prompt="Root file name for calibration lamp file"}
file	compdir="tresdata$"	{prompt="Directory for calibration lamp file"}
int	plotorder=1	{prompt="Echelle order to plot (No plot if zero"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveold=yes	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, imin, impath, templist, imlist
file	imout, ims1, ims2, ims3, imsx, cdir, calref, newref, newfile
string	ims, imd, ind, croot, tsize, tbin
int	pl, n
bool	vb, dt, sv, f1, f2, f3, disp

	vb = verbose
	dt = deltemp
	sv = saveold
	imd = imdate
	ind = instdir
	ims = images
	cdir = compdir
	croot = comproot
	disp = disperse

	if (ims == "") {
	    print ("CTRES: Reduce TRES images in a different directory")
	    print ("Usage: ctres images with both amps to be corrected")
	    return
	    }

	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpreamp = fixpreamp
	tproc.normfile = normfile
	tproc.normdir = normdir
	tproc.deltemp = dt
	tproc.saveold = sv
	tproc.cosmic=no
	tproc.verbose = vb

#  Make temporary list of input files
	tempin = mktemp ("tempctresin")
	files (ims, >tempin)
	tempout = mktemp ("tempctresout")
	if (access (tempout))
	    delete (tempout)

# Copy images to current directory
	if (instdir != "") {
	    fixdir (ind)
	    impath = fixdir.newdir
	    list = tempin
	    while (fscan (list,imin) != EOF) {

# If a number is given, prepend appropriate zeroes and wildcard
		isnum (imin)
		if (isnum.fpcode > 0) {
		    n = int (imin)
		    if (n < 10)
			imin = "000" // n // ".*"
		    else if (n < 100)
			imin = "00" // n // ".*"
		    else if (n < 1000)
			imin = "0" // n // ".*"
		    else
			imin = "0" // n // ".*"
		    }

# If file extension is missing, add wildcard
		else {
		    smatch (imin, ".fits")
		    if (smatch.found == no)
			imin = imin // "*"
		    }
		if (imd != "")
		    impath = impath // imd // "/" // imin
		else
		    impath = impath // imin
		filepath (impath)
		if (!access (filepath.files)) {
		    print ("QTRES: Cannot read file ",filepath.files)
		    }
		filename (filepath.files)
		if (imd != "")
		    imout = imd // "." // filename.filename
		else
		    imout = filename.filename
		if (vb) {
		    print ("QTRES: Copying file ",filepath.files," to ", imout)
		    }
		if (!access (imout))
		    copy (impath, imout)
		}
	    else {
		imout = imin
		}

# Remove overscan, trim, and merge amplifiers of images
# and remove pixel to pixel variation from images (including IR fringing)
	    tproc (imout)
	    print (imout, >>tempout)
	    }

	ims = "@" // tempout

# Remove cosmic rays from list or list file of images
	tcosmic (ims)

# Extract spectra
	textract.aproot = ""
	textract.apdir = ""
	textract.deltemp = dt
	textract.verbose = vb
	dispcor.linearize = no
	dispcor.database = "database"
	dispcor.table = ""
	dispcor.w1 = INDEF
	dispcor.w2 = INDEF
	dispcor.dw = INDEF
	dispcor.nw = INDEF
	dispcor.log = no
	dispcor.flux = yes
	dispcor.blank = 0.
	dispcor.samedisp = no
	dispcor.global = no
	dispcor.ignoreaps = no
	dispcor.confirm = no
	dispcor.listonly = no
	dispcor.verbose = vb
	dispcor.logfile = ""

	list = tempout
	while (fscan (list,imout) != EOF) {

	    textract (imout)

	    if (!disp)
		next

# Figure out aperture size, binning, and fibers present
	    trsfib (imout, verbose-, debug-)
	    tsize = trsfib.apsize
	    tsize = ""
	    tbin = trsfib.binning
	    if (trsfib.fib1 != "OFF")
		f1 = yes
	    else
		f1 = no
#	    if (trsfib.fib2 != "OFF")
#		f2 = yes
#	    else
		f2 = no
	    if (trsfib.fib3 != "OFF")
		f3 = yes
	    else
		f3 = no
	    fileroot (imout)
	    imroot = fileroot.rootname

	    if (f1) {
		ims1 = imroot // "_fib1.ec.fits"
		calref = cdir//croot//tsize//tbin//"1"//".ec.fits"
		newref = croot//tsize//tbin//"1"//".ec.fits"
		if (!access (newref)) {
		    copy (calref, newref)
		    }
		calref = cdir//"database/ec"//croot//tsize//tbin//"1"//".ec"
		newref = "database/ec"//croot//tsize//tbin//"1"//".ec"
		if (!access (newref)) {
		    copy (calref, newref)
		    }
		calref = croot//tsize//tbin//"1"//".ec"
		hedit (ims1, "REFSPEC1", calref, add+, verify-, show=vb, update+)
		imsx = imroot // "_fib1_disp.ec.fits"
		dispcor (ims1, imsx, linearize=no)
		if (sv) {
		    if (!access ("Nodisp/")) {
			mkdir ("Nodisp")
			}
		    newfile = "Nodisp/" // ims1
		    if (access (newfile)) {
			imdelete (newfile)
			}
		    movefiles (ims1, "Nodisp")
		    if (vb) {
			print ("QTRES: Original ", ims1, " => Nodisp/")
			}
		    }
		else {
		    imdelete (ims1)
		    }
		rename (imsx, ims1)
		}
	    if (f2) {
		ims2 = imroot // "_fib2.ec.fits"
		calref = cdir//croot//tsize//tbin//"2"//".ec.fits"
		newref = croot//tsize//tbin//"2"//".ec.fits"
		if (!access (newref)) {
		    copy (calref, newref)
		    if (vb) {
			print ("QTRES: Copying ",calref," to ", newref)
			}
		    }
		calref = cdir//"database/ec"//croot//tsize//tbin//"2"//".ec"
		newref = "database/ec"//croot//tsize//tbin//"2"//".ec"
		if (!access (newref)) {
		    copy (calref, newref)
		    if (vb) {
			print ("QTRES: Copying ",calref," to ", newref)
			}
		    }
		calref = croot//tsize//tbin//"2"//".ec"
		hedit (ims2, "REFSPEC1", calref, verify-, show=vb, add+, update+)
		imsx = imroot // "_fib2_disp.ec.fits"
		dispcor (ims2, imsx, linearize=no)
		if (sv) {
		    if (!access ("Nodisp/")) {
			mkdir ("Nodisp")
			}
		    newfile = "Nodisp/" // ims1
		    if (access (newfile)) {
			imdelete (newfile)
			}
		    movefiles (ims2, "Nodisp")
		    if (vb) {
			print ("HCAL: Original ", ims2, " => Nodisp/")
			}
		    }
		else {
		    imdelete (ims2)
		    }
		rename (imsx, ims2)
		}
	    if (f3) {
		ims3 = imroot // "_fib3.ec.fits"
		calref = cdir//croot//tsize//tbin//"2"//".ec.fits"
		newref = croot//tsize//tbin//"2"//".ec.fits"
		if (!access (newref)) {
		    copy (calref, newref)
		    }
		calref = cdir//"database/ec"//croot//tsize//tbin//"2"//".ec"
		newref = "database/ec"//croot//tsize//tbin//"2"//".ec"
		if (!access (newref)) {
		    copy (calref, newref)
		    }
		calref = croot//tsize//tbin//"2"//".ec"
		hedit (ims1, "REFSPEC1", calref, verify-, add+, show=vb, update+)
		imsx = imroot // "_fib2_disp.ec.fits"
		dispcor (ims3, imsx, linearize=no)
		if (sv) {
		    if (!access ("Nodisp/")) {
			mkdir ("Nodisp")
			}
		    newfile = "Nodisp/" // ims1
		    if (access (newfile)) {
			imdelete (newfile)
			}
		    movefiles (ims3, "Nodisp")
		    if (vb) {
			print ("HCAL: Original ", ims3, " => Nodisp/")
			}
		    }
		else {
		    imdelete (ims3)
		    }
		rename (imsx, ims3)
		}

# Plot spectra if desired
	    if (pl > 0) {
		if (f1)
		    splot (ims1, pl)
		if (f2)
		    splot (ims2, pl)
		if (f3)
		    splot (ims3, pl)
		}
	    }

#  Delete temporary list files
	if (dt) {
	    delete (tempin)
	    delete (tempout)
	    }

end

# Jan 17 2008	New task for quick look with cosmic ray removal
# Jan 25 2008	Access data by sequence number as well as file name
# Jan 25 2008	Add wavelength calibration and plotting
