# TMAKEFLAT -- Run apflatten to make a 2-d pixel normalization file
# By Doug Mink, SAO Telescope Data Center
# January 28, 2008

procedure tmakeflat (flatfile)

string	flatfile = ""	{prompt="Input multispec images to combine and fit"}
file	imdir=""	{prompt="Directory for flat and norm files"}

string	apertures="1-104"	{prompt="Apertures to flatten"}
string	apref=""	{prompt="Aperture reference file"}
int	norders=51		{prompt="Number of orders to use"}
int	aporder=5		{prompt="Order to fit apertures"}
file	apdir=""	{prompt="Aperture reference directory"}
bool	verbose = yes	{prompt="Log progress (y or n)"}

begin
	bool vb
	file imflat, imnorm, idir, aref, adir, normfile
	int nap
	string history, ffile

	ffile = flatfile
	fixdir (imdir)
	idir = imdir
	imflat = idir // ffile
	vb = verbose
	nap = norders

	fileroot (ffile)
	normfile = fileroot.root // ".flat.fits"
	imnorm = idir // normfile

	aref = apref
	if (aref == "") {
	    aref = imflat
	    adir = idir
	    }
	else {
	    fixdir (apdir)
	    adir = fixdir.newdir
	    }

# Add extension to input file, if one is not present
	smatch (ffile, ".fits")
	if (!smatch.found) {
	    imflat = imflat // ".fits"
	    }

	if (vb) {
	    print ("TMAKEFLAT: ", imflat," -> ",imnorm)
	    }

	if (access (imnorm))
	    delete (imnorm)

# Find apertures (=orders) in image
	smatch (imflat, "12")
	if (smatch.found) {
	    apfind (imflat, nap*2)
	    }
	else {
	    apfind (imflat, nap)
	    }
	apedit (imflat)
	aptrace (imflat)

# Run apflatten
	apflatten.apertures = apertures
	apflatten.references = adir // aref
	apflatten.interactive = yes
	apflatten.find = no
	apflatten.recenter = no
	apflatten.resize = no
	apflatten.edit = no
	apflatten.trace = no
	apflatten.fittrace = no
	apflatten.flatten = yes
	apflatten.fitspec = yes
	apflatten.line = INDEF
	apflatten.nsum = 1
	apflatten.threshold = 10.
	apflatten.pfit = "fit1d"
	apflatten.clean = no
	apflatten.saturation = INDEF
	apflatten.readnoise = "2.8"
	apflatten.gain = "1."
	apflatten.lsigma = 4.
	apflatten.usigma = 4.
	apflatten.function = "spline3"
	apflatten.order = aporder
	apflatten.sample = "*"
	apflatten.naverage = 1
	apflatten.niterate = 0
	apflatten.low_reject = 3.
	apflatten.high_reject = 3.
	apflatten.grow = 0.
	apflatten (imflat, imnorm)

# Add history line including time to normalization file
	gettime ()
	history = gettime.date // " " // gettime.time // " order " // aporder // " fit to "// imflat
        hedit (imnorm, "PIXFLAT", history, add+, verify-, update+, show-)
	if (vb)
	    print ("TMAKEFLAT: ",history)

end
# Oct  2 2007	New task
# Oct 10 2007	Assign flatfile to ffile so there is only one query

# Jan 28 2008	Add aperture/order finding and tracing
