# File tres/Scripts/trsdb.cl
# By Doug Mink
# January 10, 2008

# TRSDB -- Make Starbase table of major keywords in TRES data files
#             and make lists for FLATs and COMPs

procedure trsdb (spectra)

string spectra="\*.fits"	{prompt="Files to process"}

string indir=""		{prompt="Input directory for data files"}
string outdir=""	{prompt="Output directory for tables"}
bool addfile=no		{prompt="Add to database (yes) or use old (no)"}
bool listflat=yes	{prompt="List FLAT spectra by category"}
bool listcomp=yes	{prompt="List COMP spectra by category"}
bool addfib=yes		{prompt="Add FIBi keywords to spectra (yes or no)"}
bool deltemp=yes	{prompt="Delete temporary files (yes or no"}
bool verbose=yes	{prompt="Log progress (yes or no)"}
bool debug=no		{prompt="Print debugging information (yes or no)"}

begin
    bool vb, db, dt, sv
    file imf, tempilist, tempkey
    file listfile, ddir, complist, flatlist
    file flats12,flats3,flatsb12,flatsb3,flatm12,flatm3,flatmb12,flatmb3
    file comps12,comps3,compsb12,compsb3,compm12,compm3,compmb12,compmb3
    string fib1, fib2, fib3, af, aff, afb, afs, afx
    int ns, nspec
    string args, specs
    string filename, imtype, peri, ccdsum, fibsize, osswitch, comptime
    string dline1, dline2
    char tchar

    vb = verbose
    db = debug
    dt = deltemp
    fixdir (outdir)
    ddir = fixdir.newdir
    specs = spectra

# Make sure output directory is there
    if (vb && ddir != "") {
	print ("TRESDB: DestDir = ", ddir)
	}
    if (ddir != "" &&  ddir != "" && !access (ddir)) {
	mkdir (ddir)
	}

#  Make temporary list of input spectrum files
    tempilist = ddir // mktemp ("tempilist")
    tchar = substr (specs,1,1)
    if (tchar == '@') {
	listfile = substr (specs,2,100)
	copy (listfile, tempilist)
	}
    else {
	files (specs, >tempilist)
	}
    dline1 = "filename     	type	object	peri	bin	fibsize	ossw	exptime	cmptime	fib1	fib2	fib3"
    dline2 = "-----------	-------	------	----	---	-------	----	-------	-------	----	----	----"
    
# Generate master table of object spectrum keywords or use existing one
    tempkey = ddir // "speckey.db"
    if (!access (tempkey) || addfile) {
	if (listflat) {
	    flats12 = "flats12.list"
	    flats3 = "flats3.list"
	    flatsb12 = "flatsb12.list"
	    flatsb3 = "flatsb3.list"
	    flatm12 = "flatm12.list"
	    flatm3 = "flatm3.list"
	    flatmb12 = "flatmb12.list"
	    flatmb3 = "flatmb3.list"
	    if (!addfile) {
		if (access (flats12))
		    delete (flats12)
		if (access (flats3))
		    delete (flats3)
		if (access (flatsb12))
		    delete (flatsb12)
		if (access (flatsb3))
		    delete (flatsb3)
		if (access (flatm12))
		    delete (flatm12)
		if (access (flatm3))
		    delete (flatm3)
		if (access (flatmb12))
		    delete (flatmb12)
		if (access (flatmb3))
		    delete (flatmb3)
		}
	    }
	if (listcomp) {
	    comps12 = "comps12.list"
	    comps3 = "comps3.list"
	    compsb12 = "compsb12.list"
	    compsb3 = "compsb3.list"
	    compm12 = "compm12.list"
	    compm3 = "compm3.list"
	    compmb12 = "compmb12.list"
	    compmb3 = "compmb3.list"
	    if (!addfile) {
		if (access (comps12))
		    delete (comps12)
		if (access (comps3))
		    delete (comps3)
		if (access (compsb12))
		    delete (compsb12)
		if (access (compsb3))
		    delete (compsb3)
		if (access (compm12))
		    delete (compm12)
		if (access (compm3))
		    delete (compm3)
		if (access (compmb12))
		    delete (compmb12)
		if (access (compmb3))
		    delete (compmb3)
		}
	    }

	trsfib.addfib = addfib
	trsfib.verbose = debug
	trsfib.debug = debug

	if (db)
	    print ("TRESDB: Making master keyword file ",tempkey)
	ns = 0
	if (!addfile) {
	    print (dline1, >tempkey)
	    print (dline2, >>tempkey)
	    }
	if (vb) {
	    print (dline1)
	    print (dline2)
	    }
	list = tempilist
	while (fscan (list, imf) != EOF) {
	    filename = imf
	    trsfib (filename)
	    if (debug)
		dpar ("trsfib")
	    fib1 = trsfib.fib1
	    fib2 = trsfib.fib2
	    fib3 = trsfib.fib3
	    ns = ns + 1
	    printf ("%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s\n", filename, trsfib.imtype, trsfib.object, trsfib.peri, trsfib.bin1, trsfib.fibsize, trsfib.osswitch, trsfib.exptime, trsfib.comptime, fib1, fib2, fib3, >>tempkey)
	    if (vb)
	    printf ("%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s\n", filename, trsfib.imtype, trsfib.object, trsfib.peri, trsfib.bin1, trsfib.fibsize, trsfib.osswitch, trsfib.exptime, trsfib.comptime, fib1, fib2, fib3)

	    if (listflat) {
		flatlist = "flat" // trsfib.apsize // trsfib.binning
		if (fib1 == "FLAT")
		    flatlist = flatlist // "12.list"
		else if (fib3 == "COMP")
		    flatlist = flatlist // "3.list"
		print (filename, >> flatlist)
		}
	    if (listcomp) {
		complist = "comp" // trsfib.apsize // trsfib.binning
		if (fib1 == "COMP")
		    complist = complist // "12.list"
		else if (fib3 == "COMP")
		    complist = complist // "3.list"
		print (filename, >> complist
		}
	    }
	}
    else if (vb) {
	print ("TRESDB: Using existing master keyword file ", tempkey)
	concatenate (tempkey)
	}

    if (deltemp && access (tempilist))
	delete (tempilist)

end

# Oct 12 2007	New program only making database file

# Jan 10 2008	Change name from tresdb to trsdb
# Jan 10 2008	Change calls to tresfib to trsfib
